const glob = require('glob');
const fs = require('fs');
const path = require('path');

const pattern = process.argv[2];
const mappingsConfig = process.argv[3];

// parse source-roots option
const mappings = new Map();
for (const path of mappingsConfig.split(',')) {
  const split = path.split('/');
  const key = split.pop();
  mappings.set(key, split);
}

function findDifferentCaseFile(directory, name) {
  const listing = fs.readdirSync(directory);
  for (let filename of listing) {
    console.log(filename);
  }
  return undefined;
}

glob(pattern, {}, function(err, files) {
  if (!err) {
    process.stdout.write(`Checking ${files.length} files...\n`);
    let fixed = 0;
    files.forEach(function(filename) {
      const contents = fs.readFileSync(filename, 'utf-8');
      const lines = contents.split('\n');
      let needWrite = false;
      for (let i = 0, ii = lines.length; i< ii; ++i) {
        const line = lines[i];
        const match = line.match(/goog\.require\('.*\.([^']*)'\);$/);
        if (match && match.length) {
          const split = match.split('.');
          const key = split.shift();
          const name = split.pop();
          const dependencyRoot = mappings.get(key);
          const dependencyDirectory = `${dependencyRoot}/${split.join('/')}`;
          const expectedFilename = `${dependencyDirectory}/${name}.js`;
          if (!fs.existsSync(expectedFilename)) {
            // The file does not exist, searching for a file with a different case
            const alternateFilename = findDifferentCaseFile(dependencyDirectory, name);
            if (!alternateFilename) {
              throw new Error('Could not find a file with alternate case for dependency ' + match);
            }
            needWrite = true;
            lines[i] = line.replace(
              `goog.require(${match})`,
              `goog.require(${key}.${split.join('.')}.${alternateFilename.replace(/\.js$/, '')}`);
          }
        }
      }
      if (needWrite) {
        ++
      }
    });
    process.stdout.write(`Renamed ${fixed} files.\n`);
  } else {
    process.stdout.write(err.message);
    process.exit(1);
  }
});
